#!/bin/bash
DEF_DIRECTORY=/opt/Impact_Acquire

if [ "$MVIMPACT_ACQUIRE_DATA_DIR" == "/opt/mvIMPACT_Acquire/data" ] || [[ -z "$MVIMPACT_ACQUIRE_DATA_DIR" ]]; then
    DEF_DATA_DIRECTORY="$DEF_DIRECTORY/data"
else
    DEF_DATA_DIRECTORY="$MVIMPACT_ACQUIRE_DATA_DIR"
fi

PRODUCT=mvBlueFOX
TARNAME=mvBlueFOX
PACKAGENAME=mvBlueFOX
USE_DEFAULTS=NO
UNATTENDED_INSTALLATION=NO
REMOVE_ONLY=NO
MINIMAL_INSTALLATION=NO
OS_NAME="unknown"
OS_VERSION="unknown"
OS_CODENAME="unknown"
KERNEL_VERSION="unknown"
REPOSITORY_ACCESS_AVAILABLE=FALSE

ARM_ARCHITECTURE="$(uname -m)"
APT_GET_EXTRA_PARAMS=""

# Define a variable for the ErrorCount and WarningCount
ERROR_NUMBER=0
WARNING_NUMBER=0

# Define variables for colorized bash output
# Foreground
red=$(tput setaf 1)
yellow=$(tput setaf 3)
green=$(tput setaf 10)
bold=$(tput bold)
# Background
reset=$(tput sgr0)

UDEVADM=$(command -v udevadm)

# Define the users real name if possible, to prevent accidental root ownership if script is invoked with sudo
if [ "$(command -v logname)" == "" ]; then
    USER=$(whoami)
else
    if [ "$(logname 2>&1 | grep -c logname:)" == "1" ]; then
        USER=$(whoami)
    else
        USER=$(logname)
    fi
fi

# If user is root, then sudo shouldn't be used
if [ "$USER" == "root" ]; then
    SUDO=
else
    SUDO=$(command -v sudo)
fi

# Determine OS Name, version and Kernel version
function check_distro_and_version()
{
    if [ -f /etc/fedora-release ]; then
        OS_NAME='Fedora'
        OS_VERSION=$(sed </etc/redhat-release s/.*release\ // | sed s/\ .*//)
    elif [ -f /etc/redhat-release ]; then
        OS_NAME='RedHat'
        OS_VERSION=$(sed </etc/redhat-release s/.*release\ // | sed s/\ .*//)
    elif [ -f /etc/centos-release ]; then
        OS_NAME='CentOS'
        OS_VERSION=$(sed </etc/centos-release s/.*release\ // | sed s/\ .*//)
    elif [ -f /etc/SuSE-release ]; then
        OS_NAME='SuSE'
        OS_VERSION=$(tr </etc/SuSE-release "\n" ' ' | sed s/.*=\ //)
    elif [ -f /etc/SUSE-brand ]; then
        OS_NAME='SuSE'
        OS_VERSION=$(tr </etc/SUSE-brand "\n" ' ' | sed s/.*=\ //)
    elif [ -f /etc/mandrake-release ]; then
        OS_NAME='Mandrake'
        OS_VERSION=$(sed </etc/mandrake-release s/.*release\ // | sed s/\ .*//)
    elif [ -x /usr/bin/lsb_release ]; then
        OS_NAME="$(lsb_release -is)" #Ubuntu
        OS_VERSION="$(lsb_release -rs)"
        OS_CODENAME="$(lsb_release -cs)"
    elif [ -f /etc/debian_version ]; then
        OS_NAME="Debian"
        OS_VERSION="$(</etc/debian_version)"
    fi
    KERNEL_VERSION=$(uname -r)
    if [[ -v JETSON_KERNEL ]] && [[ -z "$JETSON_KERNEL" ]]; then
        JETSON_KERNEL=$(uname -r | grep tegra)
    fi
}

function checkAPTGETRepositoryAccess()
{
    hosts=$(apt-cache policy | grep -Eo 'http://[^ >]+' | sed -e 's/^http:\/\///g' -e 's/\/.*//' | sort -u)
    host=($hosts)
    HOSTS_TO_BE_PINGED_COUNT=${#host[@]}
    for entry in "${host[@]}"; do
        if [[ $entry != *"microsoft"* ]] && [[ $entry != *"ppa"* ]]; then
            ping -c2 "$entry" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                ((PINGED_SUCCESSFULLY = PINGED_SUCCESSFULLY + 1))
            fi
        else
            ((HOSTS_TO_BE_PINGED_COUNT = HOSTS_TO_BE_PINGED_COUNT - 1))
        fi
    done
    status=$((PINGED_SUCCESSFULLY - HOSTS_TO_BE_PINGED_COUNT))
    return $status
}

function checkRepositoryAccess()
{
    echo
    echo "Checking repository access. This might take a moment... "
    if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ]; then
        if which apt-get >/dev/null 2>&1; then
            checkAPTGETRepositoryAccess
            if [ $? == 0 ]; then
                echo 'Success: apt-get has repository access.'
            else
                return 1
            fi
        fi
    fi
    return 0
}

function couldNotInstallMessage()
{
    printf "Could not install the selected package(s) >%s<. You may need to install it manually" "$1"
    ((WARNING_NUMBER = WARNING_NUMBER + 1))
}

function couldNotFindPkgManagerMessage()
{
    printf "Could not find the package manager of the system; please install >%s< manually." "$1"
    ((WARNING_NUMBER = WARNING_NUMBER + 1))
}

function installPackages()
{
    # returns 0 on success
    pkgs=("$@")
    # package manager extra parameters
    APT_GET_EXTRA_PARAMS=" -y"

    echo "Installing ${pkgs[*]}..."
    if [ "$OS_NAME" == "Ubuntu" ] || [ "$OS_NAME" == "Debian" ] || [ "$OS_NAME" == "Linuxmint" ]; then
        if which apt-get >/dev/null 2>&1; then
            INSTALLATION_RESULT=$($SUDO apt-get $APT_GET_EXTRA_PARAMS -q install "${pkgs[@]}")
            if [ "$INSTALLATION_RESULT" != 0 ]; then
                couldNotInstallMessage "${pkgs[@]}"
                return 1
            fi
        else
            couldNotFindPkgManagerMessage "${pkgs[@]}"
            return 1
        fi
    else
        return 1
    fi
    # success
    return 0
}

function createSoftlink()
{
    if [ ! -e "$1/$2" ]; then
        echo "Error: File ""$1"/"$2"" does not exist, softlink cannot be created! "
        exit 1
    fi
    if [ -e "$1/$3" ]; then
        rm -rf "$1/$3" >/dev/null 2>&1
    fi
    if ! [ -L "$1/$3" ]; then
        ln -fs "$2" "$1/$3" >/dev/null 2>&1
        if ! [ -L "$1/$3" ]; then
            $SUDO ln -fs "$2" "$1/$3" >/dev/null 2>&1
            if ! [ -L "$1/$3" ]; then
                echo "Error: Could not create softlink $1/$3, even with sudo!"
                exit 1
            fi
        fi
    fi
}

function remove_previous_installation()
{
    $SUDO rm -f /usr/bin/DeviceConfigure >/dev/null 2>&1
    $SUDO rm -f /usr/bin/IPConfigure >/dev/null 2>&1
    $SUDO rm -f /usr/bin/ImpactControlCenter >/dev/null 2>&1
    $SUDO rm -f /etc/ld.so.conf.d/acquire.conf >/dev/null 2>&1
    $SUDO rm -f /etc/ld.so.conf.d/genicam.conf >/dev/null 2>&1
    $SUDO rm -f /etc/profile.d/acquire.sh >/dev/null 2>&1
    $SUDO rm -f /etc/profile.d/genicam.sh >/dev/null 2>&1
    $SUDO rm -f /etc/udev/rules.d/51-mvbf.rules >/dev/null 2>&1
    $SUDO rm -f /etc/udev/rules.d/52-U3V.rules >/dev/null 2>&1
    $SUDO rm -f /etc/udev/rules.d/52-mvbf3.rules >/dev/null 2>&1
    $SUDO rm -f /etc/sysctl.d/62-buffers-performance.conf >/dev/null 2>&1
    $SUDO rm -f /etc/security/limits.d/acquire.conf >/dev/null 2>&1
    # unload and delete any older version of the BN kernel module and then delete the udev rules
    BN_MODNAME="mvpci"
    MOD_STATUS=$(lsmod | grep -w $BN_MODNAME)
    if [ "x${MOD_STATUS}" != "x" ]; then
        $SUDO rmmod $BN_MODNAME
    fi
    $SUDO rm -f /lib/modules/"$(uname -r)"/kernel/misc/${BN_MODNAME}.ko
    $SUDO rm -f /etc/udev/rules.d/51-udev-pcie.rules >/dev/null 2>&1
    $SUDO rm -f /usr/local/bin/make_device.sh 2>&1
    $SUDO rm -f /usr/local/bin/device_namer.sh 2>&1
    $SUDO rm -rf /opt/mvIMPACT_Acquire/data >/dev/null 2>&1
    $SUDO rm -rf "$1" >/dev/null 2>&1
    return $?
}

# Print out ASCII-Art Logo.
clear
echo ""
echo ""
echo ""
echo ""
echo "                                ===      ===     MMM~,    M                     "
echo "                                 ==+     ==      M   .M   M                     "
echo "                                 .==    .=+      M    M.  M   M    MM   ~MMM    "
echo "                                  ==+   ==.      MMMMM.   M   M    MM  M:   M   "
echo "             ..                    .== ,==       M   =M   M   M    MM +MMMMMMM  "
echo "   MMMM   DMMMMMM      MMMMMM       =====        M    MM  M   M    MM 7M        "
echo "   MMMM MMMMMMMMMMM :MMMMMMMMMM      ====        M    M+  M   MM  DMM  MM   ,   "
echo "   MMMMMMMMMMMMMMMMMMMMMMMMMMMMM                 IMM+''   M    .M:       =MI    "
echo "   MMMMMMM   .MMMMMMMM    MMMMMM                                                "
echo "   MMMMM.      MMMMMM      MMMMM                                                "
echo "   MMMMM       MMMMM       MMMMM                 MMMMMM    MMMMM    MM.   M     "
echo "   MMMMM       MMMMM       MMMMM                 M       MM    .MM  .M: .M      "
echo "   MMMMM       MMMMM       MMMMM                 M      .M       M~  .M~M       "
echo "   MMMMM       MMMMM       MMMMM                 MMMMMM MM       MD   +MM       "
echo "   MMMMM       MMMMM       MMMMM                 M       M       M    M MM      "
echo "   MMMMM       MMMMM       MMMMM                 M       MM     MM  :M. .M8     "
echo "   MMMMM       MMMMM       MMMMM                 M        .MMMMM    M     MD    "
echo "   MMMMM       MMMMM       MMMMM                                                "

echo ""
echo "=============================================================================="
sleep 1

# Analyze the command line arguments and react accordingly
FORCE_REPOSITORY_USAGE=NO
PATH_EXPECTED=NO
SHOW_HELP=NO
while [[ $# -gt 0 ]]; do
    if [ "$1" == "-f" ] || [ "$1" == "--force" ]; then
        FORCE_REPOSITORY_USAGE=YES
    elif [ "$1" == "-h" ] || [ "$1" == "--help" ]; then
        SHOW_HELP=YES
        break
    elif [[ ("$1" == "-u" || "$1" == "--unattended") && "$PATH_EXPECTED" == "NO" ]]; then
        UNATTENDED_INSTALLATION=YES
    elif [[ ("$1" == "-m" || "$1" == "--minimal") && "$PATH_EXPECTED" == "NO" ]]; then
        MINIMAL_INSTALLATION=YES
    elif [[ ("$1" == "-r" || "$1" == "--remove") && "$PATH_EXPECTED" == "NO" ]]; then
        REMOVE_ONLY=YES
    elif [[ ("$1" == "-p" || "$1" == "--path") && "$PATH_EXPECTED" == "NO" ]]; then
        if [ "$2" == "" ]; then
            echo
            echo "WARNING: Path option used with no defined path, will use: $DEF_DIRECTORY directory"
            echo
            SHOW_HELP=YES
            break
        else
            PATH_EXPECTED=YES
        fi
    elif [ "$PATH_EXPECTED" == "YES" ]; then
        DEF_DIRECTORY=$1
        PATH_EXPECTED=NO
    else
        echo 'Please check your syntax and try again!'
        SHOW_HELP=YES
    fi
    shift
done

if [ "$SHOW_HELP" == "YES" ]; then
    echo
    echo "Installation script for the $PRODUCT driver."
    echo
    echo "Default installation path: $DEF_DIRECTORY"
    echo "Usage:                     ./${0##*/} [OPTION] ... "
    echo "Example:                   ./${0##*/} -p /myPath -u"
    echo
    echo "Arguments:"
    echo "-f --force                 Force the installation script to use the package manager of the system to install"
    echo "                           required packages if the installation script reports inaccessible repositories."
    echo "                           e.g. a package source on local media."
    echo "-h --help                  Display this help."
    echo "-p --path                  Set the directory where the files shall be installed."
    echo "-u --unattended            Unattended installation with default settings. By using"
    echo "                           this parameter you explicitly accept the EULA."
    echo "-m --minimal               Minimal installation. No tools or samples will be built, and"
    echo "                           no automatic configuration and/or optimizations will be done."
    echo "                           By using this parameter you explicitly accept the EULA."
    echo "-r --remove                Remove the previously installed version for ALL devices, if found."
    echo
    echo "Note:"
    echo "                           It is possible as well to combine unattended and minimal installation."
    echo "                           Combines the use of default settings from the 'minimal' installation with"
    echo "                           not building tools and samples."
    echo
    exit 1
fi

if [ "$REMOVE_ONLY" == "YES" ] && [ "$MVIMPACT_ACQUIRE_DIR" = "" ]; then
    echo "Option to remove the previous installation chosen but no installation found!"
    echo "Quit!"
    exit 1
fi

if [ "$UNATTENDED_INSTALLATION" == "YES" ]; then
    echo
    echo "Unattended installation requested, no user interaction will be required and the"
    echo "default settings will be used."
    echo
    USE_DEFAULTS=YES
fi

if [ "$MINIMAL_INSTALLATION" == "YES" ]; then
    echo
    echo "Minimal installation requested, no user interaction will be required, no tools or samples"
    echo "will be built and no automatic configurations or optimizations will be done."
    echo
    USE_DEFAULTS=YES
fi

# Get some details about the system
check_distro_and_version

# Get driver name, version, file. In case of multiple *.tgz files in the folder select the newest version.
if [ "$(ls | grep -c "$PACKAGENAME"'.*\.tgz')" != "0" ]; then
    TARNAME=$(ls "$PACKAGENAME"*.tgz | tail -1 | sed -e s/\\.tgz//)
    if [ "$(echo "$TARNAME" | grep -c ARMhf)" != "0" ]; then
        TARGET="ARMhf"
        TARGET_UNCAPITALIZED="armhf"
    elif [ "$(echo "$TARNAME" | grep -c ARM64)" != "0" ]; then
        TARGET="ARM64"
        TARGET_UNCAPITALIZED="arm64"
    else
        echo "Error: Could not determine target architecture from filename."
        echo "In case the file has been renamed, please revert to original name."
        echo "Terminating installation script now..."
        echo
        exit 1
    fi
    TARFILE="$(ls "$PACKAGENAME"*.tgz | tail -1)"
    VERSION="$(ls "$PACKAGENAME"*.tgz | tail -1 | sed -e s/\\"$PACKAGENAME"// | sed -e s/\\-"$TARGET"_gnu.*-// | sed -e s/\\.tgz//)"
    ACT=mvIMPACT_Acquire-$TARGET-$VERSION
    ACT2=$ACT
fi

# Get the source directory (the directory where the files for the installation are) and cd to it
# (The script file must be in the same directory as the source TGZ) !!!
if command -v dirname >/dev/null; then
    SCRIPTSOURCEDIR="$(dirname $(realpath "$0"))"
fi
if [ "$SCRIPTSOURCEDIR" != "$PWD" ]; then
    if [ "$SCRIPTSOURCEDIR" == "" ] || [ "$SCRIPTSOURCEDIR" == "." ]; then
        SCRIPTSOURCEDIR="$PWD"
    fi
    cd "$SCRIPTSOURCEDIR" || exit
fi

# Set variables for GenICam and mvIMPACT_acquire for later use
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
    ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf.d/acquire.conf
else
    ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf
fi

# Make sure the environment variables are set at the next boot as well
if grep -q '/etc/profile.d/' /etc/profile; then
    ACQUIRE_EXPORT_FILE=/etc/profile.d/acquire.sh
else
    ACQUIRE_EXPORT_FILE=/etc/profile
fi

# A quick check whether the Version has a correct format (due to other files being in the same directory..?)
if [ "$(echo "$VERSION" | grep -c '^[0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{1,2\}')" == "0" ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${red}  ABORTING: Script could not determine a valid $PRODUCT *.tgz file!  "
    echo "${reset}-----------------------------------------------------------------------------------"
    echo "  This script could not extract a valid version number from the *.tgz file"
    echo "  This script determined $TARFILE as the file containing the installation data."
    echo "  It is recommended that only this script and the correct *.tgz file reside in this directory."
    echo "  Please remove all other files and try again."
    exit 1
fi

# A quick check whether the user has been determined
if [ "$USER" == "" ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${red}  ABORTING: Script could not determine a valid user!  ${reset}"
    echo "-----------------------------------------------------------------------------------"
    echo "  This script could not determine the user of this shell"
    echo "  Please make sure this is a valid login shell!"
    exit 1
fi

YES_NO=
# Ask whether to use the defaults or proceed with an interactive installation
if [ "$UNATTENDED_INSTALLATION" == "NO" ] && [ "$MINIMAL_INSTALLATION" == "NO" ]; then
    echo
    echo "Would you like this installation to run in unattended mode?"
    echo "Using this mode you explicitly agree to the EULA(End User License Agreement)!"
    echo "No user interaction will be required, and the default settings will be used!"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    read YES_NO
else
    YES_NO=""
fi
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    USE_DEFAULTS=NO
else
    USE_DEFAULTS=YES
fi

YES_NO=
# Check if package manager has access to repositories
if [ "$MINIMAL_INSTALLATION" == "NO" ] && [ "$REMOVE_ONLY" != "YES" ]; then
    checkRepositoryAccess
    if [ $? == 0 ]; then
        REPOSITORY_ACCESS_AVAILABLE="YES"
    else
        echo "-----------------------------------------------------------------------------------"
        echo "${yellow}  WARNING: There seems to be no repository access!${reset}                "
        echo "-----------------------------------------------------------------------------------"
        echo "  This script could not get access to the system's package repositories."
        echo "  If additional packages are requiered during installation e.g.: g++,"
        echo "  wxWidgets, expat, FLTK, ... some features might not be available."
        echo
        echo "  In certain cases this might be a result of specific firewall settings or local   "
        echo "  package repositores. It is possible to let the script try to use those package   "
        echo "  repositories even if the access could not be verified.                           "
        echo
        echo "  Would you like this installation to force the usage of the package manager to    "
        echo "  install some additional required packages?"
        echo "  Hit 'y' + <Enter> for 'yes', or just <Enter> for 'no'."

        if [ "$USE_DEFAULTS" == "NO" ]; then
            read YES_NO
        else
            YES_NO=""
        fi
        if [ "$YES_NO" == "y" ] || [ "$YES_NO" == "Y" ]; then
            FORCE_REPOSITORY_USAGE="YES"
        fi
    fi
fi

YES_NO=
# Here we will ask the user if we shall start the installation process
echo
echo "-----------------------------------------------------------------------------------"
echo "${bold}Host System:${reset}"
echo "-----------------------------------------------------------------------------------"
echo
echo "${bold}OS:                             ${reset} $OS_NAME"
echo "${bold}OS Version:                     ${reset} $OS_VERSION"
echo "${bold}OS Codename:                    ${reset} $OS_CODENAME"
echo "${bold}Kernel:                         ${reset} $KERNEL_VERSION"
echo "${bold}Platform:                       ${reset} $TARGET"
echo
if [ "$REMOVE_ONLY" == "YES" ]; then
    echo "${bold}Installation directory to be removed: ${reset} $MVIMPACT_ACQUIRE_DIR"
else
    echo "-----------------------------------------------------------------------------------"
    echo "${bold}Configuration:${reset}"
    echo "-----------------------------------------------------------------------------------"
    echo
    echo "${bold}Installation for user:            ${reset} $USER"
    echo "${bold}Installation directory:           ${reset} $DEF_DIRECTORY"
    echo "${bold}Data directory:                   ${reset} $DEF_DATA_DIRECTORY"
    echo "${bold}Source directory:                 ${reset} $(echo "$SCRIPTSOURCEDIR" | sed -e 's/\/\.//')"
    echo "${bold}Version:                          ${reset} $VERSION"
    echo "${bold}TAR-File:                         ${reset} $TARFILE"
fi
echo
echo "${bold}ldconfig:"
echo "${bold}mvIMPACT_acquire:                 ${reset} $ACQUIRE_LDSOCONF_FILE"
echo
echo "${bold}Exports:"
echo "${bold}mvIMPACT_acquire:                 ${reset} $ACQUIRE_EXPORT_FILE"

echo
echo "-----------------------------------------------------------------------------------"
echo
echo "Do you want to continue (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$USE_DEFAULTS" == "NO" ]; then
    read YES_NO
else
    YES_NO=""
fi
echo

# If the user is choosing no, we will abort the installation, else we will start the process.
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    echo "Quit!"
    exit
fi

if [ "$REMOVE_ONLY" == "YES" ]; then
    remove_previous_installation "$MVIMPACT_ACQUIRE_DIR"
    result=$?
    if [ $result == 0 ]; then
        echo "Previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR) removed successfully!"
    else
        echo "Error removing previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR)!"
    fi
    exit "$result"
fi

# End User License Agreement
YES_NO="r"
while [ "$YES_NO" == "r" ] || [ "$YES_NO" == "R" ]; do
    echo
    echo "Do you accept the End User License Agreement (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', 'r' + <Enter> to read the EULA or "
    echo "just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read YES_NO
        if [ "$YES_NO" == "r" ] || [ "$YES_NO" == "R" ]; then
            if [ "x$(command -v more)" != "x" ]; then
                EULA_SHOW_COMMAND="more -d"
            else
                EULA_SHOW_COMMAND="cat"
            fi
            tar -xzf "$SCRIPTSOURCEDIR/$TARFILE" -C /tmp mvIMPACT_Acquire-"$TARGET"-"$VERSION"/doc/EULA.txt --strip-components=2 && $EULA_SHOW_COMMAND /tmp/EULA.txt && rm /tmp/EULA.txt && sleep 1
            # clear the stdin buffer in case user spammed the Enter key
            while read -r -t 0; do read -r; done
        fi
    else
        YES_NO=""
    fi
done

# If the user is choosing no, we will abort the installation, else we continue.
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    echo "Quit!"
    exit
fi

echo
echo "-----------------------------------------------------------------------------------"
echo "${bold}BY INSTALLING THIS SOFTWARE YOU HAVE AGREED TO THE EULA(END USER LICENSE AGREEMENT)${reset}"
echo "-----------------------------------------------------------------------------------"
echo

# First of all ask whether to dispose of the old Impact Acquire installation
if [ "$MVIMPACT_ACQUIRE_DIR" != "" ]; then
    DEFAULT_KEEP="yes"
    INCOMPATIBLE_VERSION="no"
    NEW_MAJOR=$(echo "$VERSION" | cut -d . -f 1)
    # Determine the version number of the previous installation
    OLD_DEVICE_MANAGER="$(basename "$(find "$MVIMPACT_ACQUIRE_DIR"/lib/"${TARGET_UNCAPITALIZED}" -name "libmvDeviceManager.so.*.*.*" -print -quit)")"
    if [ "$OLD_DEVICE_MANAGER" != "" ]; then
        OLD_MAJOR=$(echo "$OLD_DEVICE_MANAGER" | cut -d . -f 3)
        OLD_MINOR=$(echo "$OLD_DEVICE_MANAGER" | cut -d . -f 4)
        OLD_RELEASE=$(echo "$OLD_DEVICE_MANAGER" | exec cut -d . -f 5)
        echo "Previous installation version $OLD_MAJOR.$OLD_MINOR.$OLD_RELEASE detected at: $MVIMPACT_ACQUIRE_DIR"
        if [ "$OLD_MAJOR" != "$NEW_MAJOR" ]; then
            INCOMPATIBLE_VERSION="yes"
            DEFAULT_KEEP="no"
            echo "This version is incompatible with the new installation and you must remove it!"
        fi
    fi
    echo "Do you want to keep the previous installation (default is '${DEFAULT_KEEP}')?"
    echo "If you select no, Impact Acquire will be removed for ALL installed products!"
    if [ "$INCOMPATIBLE_VERSION" == "yes" ]; then
        echo "Hit 'y' + <Enter> for 'yes', or just <Enter> for 'no'."
    else
        echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    fi
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read YES_NO
    else
        YES_NO=$DEFAULT_KEEP
    fi
    echo
    YES_NO=$(echo "$YES_NO" | tr '[:upper:]' '[:lower:]')
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "no" ]; then
        remove_previous_installation "$MVIMPACT_ACQUIRE_DIR"
        result=$?
        if [ $result == 0 ]; then
            echo "Previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR) removed successfully!"
        else
            echo "Error removing previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR)!"
            echo "$result"
        fi
        if [ "$INIT_SYSTEM" == "systemd" ] && [ -f "/etc/systemd/system/resize_usbfs_buffersize.service" ]; then
            $SUDO systemctl stop resize_usbfs_buffersize.service
            $SUDO systemctl disable resize_usbfs_buffersize.service
            $SUDO rm -f /etc/systemd/system/resize_usbfs_buffersize.service
            result=$?
            if [ $result == 0 ]; then
                echo "usbcore.usbfs_memory_mb systemd service (/etc/systemd/system/resize_usbfs_buffersize.service) disabled and removed successfully!"
            else
                echo "Error removing usbcore.usbfs_memory_mb systemd service (/etc/systemd/system/resize_usbfs_buffersize.service)!"
                echo "$result"
            fi
        fi
    else
        echo "Previous Impact Acquire installation ($MVIMPACT_ACQUIRE_DIR) NOT removed!"
        if [ "$INCOMPATIBLE_VERSION" == "yes" ]; then
            echo "Stopping the installer due to incompatible previous installation version $OLD_MAJOR.$OLD_MINOR.$OLD_RELEASE!"
            exit 1
        fi
    fi
fi

# Create the *.conf files if the system is supporting ld.so.conf.d
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
    $SUDO rm -f "$ACQUIRE_LDSOCONF_FILE"
    $SUDO touch "$ACQUIRE_LDSOCONF_FILE"
fi

# Create the export files if the system is supporting profile.d
if grep -q '/etc/profile.d/' /etc/profile; then
    $SUDO rm -f "$ACQUIRE_EXPORT_FILE"
    $SUDO touch "$ACQUIRE_EXPORT_FILE"
fi

# Check if the destination directory exist, else create it
if ! [ -d "$DEF_DIRECTORY" ]; then
    # the destination directory does not yet exist
    # first try to create it as a normal user
    mkdir -p "$DEF_DIRECTORY" >/dev/null 2>&1
    if ! [ -d "$DEF_DIRECTORY" ]; then
        # that didn't work
        # now try it as superuser
        $SUDO mkdir -p "$DEF_DIRECTORY"
        ret=$?
    fi
    if ! [ -d "$DEF_DIRECTORY" ]; then
        echo "ERROR: Could not create target directory '$DEF_DIRECTORY'."
        echo "Problem: $ret"
        echo "Maybe you specified a partition that was mounted read only?"
        echo
        exit 1
    fi
else
    echo "Installation directory already exists."
fi

# in case the directory already existed BUT it belongs to other user
$SUDO chown -R "$USER": "$DEF_DIRECTORY"

# Check the actual tarfile
if ! [ -r "$TARFILE" ]; then
    echo "ERROR: could not read '$TARFILE'."
    echo
    exit 1
fi

# needed at compile time (used during development, but not shipped with the final program)
ACT="mvIMPACT_Acquire-$VERSION.tar"

# Now unpack the tarfile into /tmp
cd /tmp || exit 1
tar xfz "$SCRIPTSOURCEDIR/$TARFILE"

# Now check if we can unpack the tar file with the device independent stuff
# this is entirely optional
if [ -r "/tmp/$ACT2" ]; then
    cd /tmp || exit 1
    #tar xvf /tmp/$ACT
    cp -r "$ACT2"/* "$DEF_DIRECTORY"
else
    echo
    echo "ERROR: Could not read: /tmp/$ACT2"
    exit
fi

# Set the necessary exports and library paths
cd "$DEF_DIRECTORY" || exit 1
if grep -q 'MVIMPACT_ACQUIRE_DIR=' "$ACQUIRE_EXPORT_FILE"; then
    echo 'MVIMPACT_ACQUIRE_DIR already defined in' "$ACQUIRE_EXPORT_FILE".
else
    $SUDO sh -c "echo 'export MVIMPACT_ACQUIRE_DIR=$DEF_DIRECTORY' >> $ACQUIRE_EXPORT_FILE"
fi

if grep -q "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED" $ACQUIRE_LDSOCONF_FILE; then
    echo "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED already defined in" $ACQUIRE_LDSOCONF_FILE.
else
    $SUDO sh -c "echo '$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED' >> $ACQUIRE_LDSOCONF_FILE"
fi

# This variable must be exported, or else ImpactControlCenter-related make problems can arise
export MVIMPACT_ACQUIRE_DIR=$DEF_DIRECTORY

# Clean up /tmp
rm -rf "/tmp/$ACT2" "/tmp/mvIMPACT_Acquire-$VERSION"

#create softlinks for the Toolkits libraries
createSoftlink $DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib $(ls $DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib | grep libexpat\.so\..*\..*\.) libexpat.so.1
createSoftlink $DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib libexpat.so.1 libexpat.so
createSoftlink $DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage/$TARGET_UNCAPITALIZED $(ls $DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage/$TARGET_UNCAPITALIZED | grep libfreeimage-3\..*\.so) libfreeimage.so.3
createSoftlink $DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage/$TARGET_UNCAPITALIZED libfreeimage.so.3 libfreeimage.so

#An important distinction has to be made here between 32bit and 64bit ARM systems
if [ $TARGET != "ARM64" ]; then
    # Since the native make target for 32bit ARM architectures can have many different values
    # (eg. armv7l, arm7ahf etc. ) and will not be 'armhf', a softlink has to be created,
    # otherwise the our apps and tools will not be able to be linked with the our libraries.
    createSoftlink $DEF_DIRECTORY/lib $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE
    createSoftlink $DEF_DIRECTORY/Toolkits/expat/bin $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE
    createSoftlink $DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE
else
    # In case of 64bit ARM architectures, they always report 'aarch64' back, which makes our lives
    # considerably easier. In this case we do not need to create softlinks, but the $ARM_ARCHITECTURE
    # needs to be overwritten so that the mvApps softlinks in /usr/bin will point to the correct path.
    ARM_ARCHITECTURE=$TARGET_UNCAPITALIZED
fi

# Update the library cache with ldconfig
$SUDO /sbin/ldconfig

if [ "$MINIMAL_INSTALLATION" == "NO" ]; then
    if {
        [ "$REPOSITORY_ACCESS_AVAILABLE" == "YES" ] || [ "$FORCE_REPOSITORY_USAGE" == "YES" ]
    } || [ "$USE_DEFAULTS" == "YES" ]; then
        ADDITIONAL_PACKAGE_TO_BE_INSTALLED=""

        # Ask whether the samples should be built natively
        if [ "$TARGET" != "armv7ahf" ]; then
            # not building native for the above platforms - this is a CROSS install and we do not have the NATIVE libmv* files!
            echo
            echo "Do you want the sample applications to be built (default is 'yes')?"
            echo "A native g++ compiler has to be present on the system!"
            echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
            if [ "$USE_DEFAULTS" == "NO" ]; then
                read YES_NO
            else
                YES_NO=""
            fi
            echo
            if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                echo 'The tools and samples will not be built.'
                echo 'To build them manually, type:'
                echo '  cd '$DEF_DIRECTORY
                echo '  make native'
                echo '  sudo /sbin/ldconfig'
                echo 'later!'
            else
                if [ "$(which g++)" != "" ]; then
                    echo "Do you want the GUI tools to be built (default is 'yes')?"
                    echo "This requires wxWidgets libraries to be present on your system."
                    echo "If they are missing, an attempt will be made to download them"
                    echo "(which may require an Internet connection!)."
                    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
                    if [ "$USE_DEFAULTS" == "NO" ]; then
                        read YES_NO
                    else
                        YES_NO=""
                    fi
                    echo
                    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                        # remove GUI apps sources since they are not needed
                        rm -rf $DEF_DIRECTORY/apps/mv*
                        rm -rf $DEF_DIRECTORY/apps/Common/FirmwareUpdate_mvHYPERION
                    else
                        if [ "$REPOSITORY_ACCESS_AVAILABLE" == "YES" ] || [ "$FORCE_REPOSITORY_USAGE" == "YES" ]; then
                            # check if wxWidgets are present else download them
                            if [ "$(wx-config --release 2>&1 | grep -c "^3.")" != "1" ] ||
                                [ "$(wx-config --libs webview 2>&1 | grep -c "webview")" != "1" ] ||
                                [ "$(wx-config --selected-config 2>&1 | grep -c "gtk3")" == "0" ]; then
                                if [ "x$(which apt-get)" != "x" ]; then
                                    echo
                                    echo "Updating file lists from repositories..."
                                    echo
                                    $SUDO apt-get update
                                    echo
                                    echo "Downloading and installing wxWidgets via apt-get..."
                                    REQUIRED_PACKAGES=(libwxgtk-webview3.2-* libwxgtk3.2-dev build-essential libgtk2.0-dev)
                                    installPackages "${REQUIRED_PACKAGES[@]}"
                                    if [ $? -ne 0 ]; then
                                        echo
                                        echo "Could not install $ADDITIONAL_PACKAGE_TO_BE_INSTALLED v3.2, falling back to v3.0 ."
                                        REQUIRED_PACKAGES=(libwxgtk-webview3.0-* libwxgtk3.0-* wx3.0-headers build-essential libgtk2.0-dev)
                                        installPackages "${REQUIRED_PACKAGES[@]}"
                                    fi
                                    echo
                                    if [ $? == 0 ] && [ "x$(which wx-config)" != "x" ]; then
                                        echo "Necessary wxWidgets libraries successfully installed!"
                                        echo
                                    else
                                        echo "wxWidgets libraries could not be downloaded and installed automatically on this system!"
                                        echo "Please either install wxWidgets libraries manually and re-run this installer script,"
                                        echo "or re-run this script and choose not to build the wxWidgets GUI tools altogether!"
                                        echo
                                        exit 1
                                    fi
                                else
                                    echo
                                    echo "Could not download wxWidgets, apt-get is missing!"
                                    echo
                                    echo "wxWidgets libraries could not be downloaded and installed automatically on this system!"
                                    echo "Please either install wxWidgets libraries manually and re-run this installer script,"
                                    echo "or re-run this script and choose not to build the wxWidgets GUI Tools altogether!"
                                    echo
                                    exit 1
                                fi
                                if [ "$(update-alternatives --list wx-config | grep -c "gtk3")" == "1" ]; then
                                    echo "Using GTK3 wxWidgets as default during this installation."
                                    $SUDO update-alternatives --set wx-config $(update-alternatives --list wx-config | grep gtk3)
                                    WXCONFIG_RESET_NEEDED="YES"
                                fi
                            fi
                            cd "$DEF_DIRECTORY" || exit 1
                            $SUDO /sbin/ldconfig
                        else
                            echo "wxWidgets libraries could not automatically download and install on this system."
                            echo "No repository seems to be available!"
                            echo
                            echo "Please make sure the repositories are available and re-run this installer script,"
                            echo "or re-run this script and choose not to build the wxWidgets GUI tools altogether!"
                            echo
                        fi
                    fi
                fi
            fi
        fi

        # In case we are compiling on a SuSE flavor, remove DeviceConfigure because wxWidgets is no longer compatible
        if [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
            if [ -d "$DEF_DIRECTORY/apps/DeviceConfigure" ] && [ -r "$DEF_DIRECTORY/apps/DeviceConfigure/Makefile" ]; then
                $SUDO rm -rf "$DEF_DIRECTORY/apps/DeviceConfigure" >/dev/null 2>&1
            fi
            if [ "$MINIMAL_INSTALLATION" == "NO" ]; then
                echo "The DeviceConfigure application will not be built because your distribution does not supply all the necessary wxWidget libraries."
            fi
        fi

        if [ "$MINIMAL_INSTALLATION" == "NO" ]; then
            echo
            echo "Do you want the tools and samples to be built (default is 'yes')?"
            echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
            if [ "$USE_DEFAULTS" == "NO" ]; then
                read YES_NO
            else
                YES_NO=""
            fi
            if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                echo
                echo "The tools and samples were not built."
                echo "To build them manually run 'make native' in $DEF_DIRECTORY"
            else
                cd "$DEF_DIRECTORY" || exit 1
                $SUDO chown -R "$USER": "$DEF_DIRECTORY"
                make native

                if [ $? -ne 0 ]; then
                    ((WARNING_NUMBER = WARNING_NUMBER + 1))
                fi

                # Shall the our tools be linked in /usr/bin?
                if [ "$GEV_SUPPORT" == "TRUE" ]; then
                    if [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                        GUI_APPS="ImpactControlCenter and IPConfigure"
                    else
                        GUI_APPS="ImpactControlCenter, IPConfigure and DeviceConfigure"
                    fi
                else
                    if [ "$OS_NAME" == "SuSE" ] || [ "$OS_NAME" == "openSUSE" ]; then
                        GUI_APPS="ImpactControlCenter and IPConfigure"
                    else
                        GUI_APPS="ImpactControlCenter and DeviceConfigure"
                    fi
                fi
                echo "Do you want to set a link to /usr/bin for $GUI_APPS (default is 'yes')?"
                echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
                if [ "$USE_DEFAULTS" == "NO" ]; then
                    read YES_NO
                else
                    YES_NO=""
                fi
                if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                    echo "Will not set any new link to /usr/bin."
                else
                    if [ -r /usr/bin ]; then
                        # Set ImpactControlCenter
                        if [ -r "$DEF_DIRECTORY"/apps/ImpactControlCenter/"$ARM_ARCHITECTURE"/ImpactControlCenter ]; then
                            $SUDO rm -f /usr/bin/ImpactControlCenter
                            $SUDO ln -s "$DEF_DIRECTORY"/apps/ImpactControlCenter/"$ARM_ARCHITECTURE"/ImpactControlCenter /usr/bin/ImpactControlCenter
                        fi
                        # Set DeviceConfigure
                        if [ -r "$DEF_DIRECTORY"/apps/DeviceConfigure/"$ARM_ARCHITECTURE"/DeviceConfigure ]; then
                            $SUDO rm -f /usr/bin/DeviceConfigure
                            $SUDO ln -s "$DEF_DIRECTORY"/apps/DeviceConfigure/"$ARM_ARCHITECTURE"/DeviceConfigure /usr/bin/DeviceConfigure
                            $SUDO ln -s "$DEF_DIRECTORY"/apps/DeviceConfigure/"$ARM_ARCHITECTURE"/DeviceConfigure /usr/bin/mvDeviceConfigure # This used to be the name of this application for versions smaller than 3.0.0
                        fi
                    fi

                fi
            fi

            # Should ImpactControlCenter check weekly for updates?
            echo "Do you want ImpactControlCenter to check for updates weekly(default is 'yes')?"
            echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
            if [ "$USE_DEFAULTS" == "NO" ]; then
                read YES_NO
            else
                YES_NO=""
            fi
            if [ ! -e ~/.ImpactControlCenter ]; then
                touch ~/.ImpactControlCenter
            fi
            if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
                if [ "$(grep -c AutoCheckForUpdatesWeekly ~/.ImpactControlCenter)" -ne "0" ]; then
                    Tweakline=$(($(grep -n "AutoCheckForUpdatesWeekly" ~/.ImpactControlCenter | cut -d: -f1))) && sed -i "$Tweakline s/.*/AutoCheckForUpdatesWeekly=0/" ~/.ImpactControlCenter
                else
                    echo "AutoCheckForUpdatesWeekly=0" >>~/.ImpactControlCenter
                fi
            else
                if [ "$(grep -c AutoCheckForUpdatesWeekly ~/.ImpactControlCenter)" -ne "0" ]; then
                    Tweakline=$(($(grep -n "AutoCheckForUpdatesWeekly" ~/.ImpactControlCenter | cut -d: -f1))) && sed -i "$Tweakline s/.*/AutoCheckForUpdatesWeekly=1/" ~/.ImpactControlCenter
                else
                    echo "[MainFrame/Help]" >>~/.ImpactControlCenter
                    echo "AutoCheckForUpdatesWeekly=1" >>~/.ImpactControlCenter
                fi
            fi
        fi
    fi
fi

echo
echo "Do you want to copy 51-mvbf.rules to /etc/udev/rules.d for non-root user support (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$USE_DEFAULTS" == "NO" ]; then
    read YES_NO
else
    YES_NO=""
fi
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    echo
    echo "To grant non-root user support,"
    echo "copy 51-mvbf.rules the file to /etc/udev/rules.d"
    echo
else
    $SUDO cp -f "$DEF_DIRECTORY"/Scripts/51-mvbf.rules /etc/udev/rules.d
fi
echo

# reload the udev rules after change.
$SUDO "$UDEVADM" control --reload-rules

# Update the library cache again.
$SUDO /sbin/ldconfig

# check if plugdev group exists and the user is member of it
if [ "$(grep -c ^plugdev: /etc/group)" == "0" ]; then
    echo "Group 'plugdev' doesn't exist, this is necessary to use USB devices as a normal user!"
    echo "Do you want to create it and add current user to 'plugdev' (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    if [ "$USE_DEFAULTS" == "NO" ]; then
        read YES_NO
    else
        YES_NO=""
    fi
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
        echo
        echo "'plugdev' will not be created and you can't run the device as non-root user!"
        echo "If you want non-root user support, you will need to create 'plugdev'"
        echo "and add these users to this group."
        ((WARNING_NUMBER = WARNING_NUMBER + 1))
    else
        $SUDO /usr/sbin/groupadd -g 46 plugdev
        $SUDO /usr/sbin/usermod -a -G plugdev "$USER"
        echo "Group 'plugdev' created and user '$USER' added to it."
    fi
else
    if [ "$(groups | grep -c plugdev)" == "0" ]; then
        echo "Group 'plugdev' exists, however user '$USER' is not a member, which is necessary to"
        echo "use USB devices. Do you want to add user '$USER' to 'plugdev' (default is 'yes')?"
        echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
        if [ "$USE_DEFAULTS" == "NO" ]; then
            read YES_NO
        else
            YES_NO=""
        fi
        if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
            echo
            echo "If you want to use USB devices you have to manually add user '$USER' to the plugdev group."
            ((WARNING_NUMBER = WARNING_NUMBER + 1))
        else
            $SUDO /usr/sbin/usermod -a -G plugdev "$USER"
            echo "User '$USER' added to 'plugdev' group."
        fi
    fi
fi
echo

# create the logs directory and set MVIMPACT_ACQUIRE_DATA_DIR.
if ! [ -d "$DEF_DATA_DIRECTORY"/logs ]; then
    mkdir -p "$DEF_DATA_DIRECTORY"/logs >/dev/null 2>&1
    if ! [ -d "$DEF_DATA_DIRECTORY"/logs ]; then
        # that didn't work, now try it as superuser
        $SUDO mkdir -p "$DEF_DATA_DIRECTORY"/logs >/dev/null 2>&1
    fi
fi

if [ -d "$DEF_DATA_DIRECTORY"/logs ]; then
    # exec (1) and write (2) are needed to create new files and reading (4) should be allowed anyway
    # therefore the permissions are set to 777, so any user is able to read and write logs
    $SUDO chmod 777 "$DEF_DATA_DIRECTORY"/logs
    mv "$DEF_DIRECTORY/apps/mvDebugFlags.mvd" "$DEF_DATA_DIRECTORY"/logs >/dev/null 2>&1
    if ! [ -r "$DEF_DATA_DIRECTORY"/logs/mvDebugFlags.mvd ]; then
        $SUDO mv "$DEF_DIRECTORY/apps/mvDebugFlags.mvd" "$DEF_DATA_DIRECTORY"/logs >/dev/null 2>&1
    fi
    if grep -q 'MVIMPACT_ACQUIRE_DATA_DIR=' "$ACQUIRE_EXPORT_FILE"; then
        echo 'MVIMPACT_ACQUIRE_DATA_DIR already defined in' "$ACQUIRE_EXPORT_FILE".
    else
        $SUDO sh -c "echo 'export MVIMPACT_ACQUIRE_DATA_DIR=$DEF_DATA_DIRECTORY' >> $ACQUIRE_EXPORT_FILE"
    fi
else
    echo "ERROR: Could not create " "$DEF_DATA_DIRECTORY"/logs " directory."
    echo 'Problem:'$?
    echo 'Maybe you specified a partition that was mounted read only?'
    echo
    exit 1
fi

# Configure the /etc/security/limits.d/acquire.conf file to be able to set thread priorities
if [ -d /etc/security/limits.d ]; then
    $SUDO touch /etc/security/limits.d/acquire.conf
    $SUDO chmod a+r /etc/security/limits.d/acquire.conf
    if [ "$(grep -c '@plugdev            -       nice            -20' /etc/security/limits.d/acquire.conf)" == "0" ]; then
        echo '@plugdev            -       nice            -20' | $SUDO tee -a /etc/security/limits.d/acquire.conf >/dev/null
    fi
    if [ "$(grep -c '@plugdev            -       rtprio          99' /etc/security/limits.d/acquire.conf)" == "0" ]; then
        echo '@plugdev            -       rtprio          99' | $SUDO tee -a /etc/security/limits.d/acquire.conf >/dev/null
    fi
else
    echo 'INFO: Directory /etc/security/limits.d is missing, Impact Acquire will not'
    echo 'be able to set thread priorities correctly. Incomplete frames may occur!!!'
    ((WARNING_NUMBER = WARNING_NUMBER + 1))
fi

# remove all example application sources in case of minimal installation
if [ "$MINIMAL_INSTALLATION" == "YES" ]; then
    $SUDO rm -rf "$DEF_DIRECTORY"/apps >/dev/null 2>&1
fi

# resetting wxWidgets configuration to the auto default
if [ "$WXCONFIG_RESET_NEEDED" ]; then
    echo "Resetting wxWidgets configuration to the auto default."
    $SUDO update-alternatives --auto wx-config
fi

echo
if [ "$ERROR_NUMBER" == 0 ] && [ "$WARNING_NUMBER" == 0 ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${green}${bold}                           Installation successful!${reset}         "
    echo "-----------------------------------------------------------------------------------"
elif [ "$ERROR_NUMBER" == 0 ] && [ "$WARNING_NUMBER" != 0 ]; then
    echo "-----------------------------------------------------------------------------------"
    echo "${yellow}${bold}                           Installation successful!${reset}        "
    echo "-----------------------------------------------------------------------------------"
    echo "                                                                                   "
    echo "  Some warnings have been issued during the installation. Typically the driver     "
    echo "  will work, but some functionality is missing e.g. some sample applications       "
    echo "  which could not be built because of missing dependencies.                        "

    echo "                                                                                   "
    echo "  Please refer to the output of the script for further details.                    "
    echo "-----------------------------------------------------------------------------------"
else
    echo "-----------------------------------------------------------------------------------"
    echo "${red}${bold}                        Installation NOT successful!${reset}          "
    echo "-----------------------------------------------------------------------------------"
    echo "                                                                                   "
    echo "  Please provide the full output of this installation script to the Balluff "
    echo "  support department (balluff@balluff.de) if the error messages shown during   "
    echo "  the installation procedure don't help you to get the driver package installed    "
    echo "  correctly!                                                                       "
    echo "-----------------------------------------------------------------------------------"
fi
echo
echo "Do you want to reboot now (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$USE_DEFAULTS" == "NO" ]; then
    read YES_NO
else
    YES_NO="n"
fi
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    echo "You need to reboot manually to complete the installation."
else
    $SUDO shutdown -r now
fi
